extern "C" {
#include <lua.h>
#include <lauxlib.h>
}

#include <ClpSimplex.hpp>

static int l_allocate(lua_State *L) {
    const int num_students = (int)luaL_checknumber(L, 1);
    const int num_subjects = (int)luaL_checknumber(L, 2);

    const int num_columns = num_students * num_subjects;
    const int num_rows = num_students + num_subjects;

    int rows[2];
    const double vals[] = {1,1};
    int objective;

    ClpSimplex  model;
    model.setLogLevel(0);
    model.setOptimizationDirection(-1); // maximize
    model.resize(num_rows, 0);
    for(int i=0; i < num_columns; i++) {
        rows[0] = (i/num_subjects);
        rows[1] = (i%num_subjects);

        lua_pushnumber(L, rows[1] + 1); // 1 based
        lua_gettable(L, rows[0] + 3); // ignore first 2 arguments

        rows[1] += num_subjects;

        if(lua_isnil(L, -1)){
            objective = 0;
        }
        else {
            objective = (int)luaL_checknumber(L,-1);
        }
        lua_pop(L,1);

        model.addColumn(2, rows, vals, 0, 1, objective);
        model.setInteger(i); // objectives are integers
    }

    for(int i=0; i < num_rows; i++) {
        model.setRowBounds(i, 0, 1);
    }

    model.dual();

    lua_newtable(L);
    const double * solution = model.primalColumnSolution();
    for (int i = 0; i < num_columns; i++) {
        if(solution[i]){
            lua_pushnumber(L, (i/num_subjects)+1);
            lua_pushnumber(L, (i%num_subjects)+1);
            lua_settable(L, -3);
        }
    }

    lua_pushboolean(L, model.isProvenOptimal());

    return 2;
}

extern "C" {
static const struct luaL_Reg l_allocator [] = {
    {"allocate", l_allocate},
    {NULL, NULL} // sentinel
};

int luaopen_allocator(lua_State *L) {
    luaL_newlib(L, l_allocator);
    return 1;
}
}
